@@ -27,9 +27,11 @@
  * all setups needs this
  */
 
+#include <jack/jack.h>
 #include <glib.h>
 
 #include "plugin_settings.h"
+#include "ui.h"
 
 typedef struct _saved_plugin saved_plugin_t;
 
@@ -39,14 +41,6 @@ struct _saved_plugin
   GSList *       midi_controls;
 };
 
-
-
-#ifdef HAVE_XML
-
-#include <jack/jack.h>
-
-#include "ui.h"
-
 typedef struct _saved_rack   saved_rack_t;
 
 struct _saved_rack
@@ -56,10 +50,4 @@ struct _saved_rack
   GSList *       plugins;
 };
 
-int ui_save_file (ui_t * ui, const char * filename);
-int ui_open_file (ui_t * ui, const char * filename);
-
-
-#endif /* HAVE_XML */
-
 #endif /* __JR_FILE_H__ */
