@@ -34,6 +34,10 @@
 #include "control_message.h"
 #include "ui.h"
 
+#ifdef HAVE_LRDF
+#include <lrdf.h>
+#endif
+
 #define TEXT_BOX_WIDTH        75
 #define TEXT_BOX_CHARS        -1
 
@@ -84,14 +88,6 @@ create_float_control (plugin_desc_t * desc, unsigned long port_index)
     }
 
 
-/*  printf ("%s: lower: %f, upper: %f, %s\n", __FUNCTION__, lower, upper,
-         LADSPA_IS_HINT_SAMPLE_RATE (desc->port_range_hints[port_index].HintDescriptor)
-          ? "sample rate adjusted" : "normal"); */
-
-  if (!(lower < upper))
-    {
-      printf ("%s: lower !< upper!\n", __FUNCTION__);
-    }
   widget = gtk_hscale_new_with_range ((gdouble) lower, (gdouble) upper, (upper - lower) / 10.0);
   gtk_scale_set_draw_value (GTK_SCALE (widget), FALSE);
   gtk_scale_set_digits (GTK_SCALE (widget), 8);
@@ -103,7 +99,7 @@ create_float_control (plugin_desc_t * desc, unsigned long port_index)
   return widget;
 }
 
-/** create a control for int ports */
+/** create a control for integer ports */
 static GtkWidget *
 create_int_control (plugin_desc_t * desc, unsigned long port_index)
 {
@@ -140,13 +136,6 @@ create_int_control (plugin_desc_t * desc, unsigned long port_index)
 
   if (!(lower < upper))
     {
-/*    printf("%s: lower !< upper! (%f, %f, %s, %s, %s)\n", __FUNCTION__,
-           desc->port_range_hints[port_index].LowerBound,
-           desc->port_range_hints[port_index].UpperBound,
-           LADSPA_IS_HINT_SAMPLE_RATE (desc->port_range_hints[port_index].HintDescriptor) ? "normal" : "sample rate",
-           LADSPA_IS_HINT_BOUNDED_BELOW (desc->port_range_hints[port_index].HintDescriptor) ? "use lower" : "do not use lower",
-           LADSPA_IS_HINT_BOUNDED_ABOVE (desc->port_range_hints[port_index].HintDescriptor) ? "use upper" : "do not use upper"
-           );*/
       if (!LADSPA_IS_HINT_BOUNDED_ABOVE
           (desc->port_range_hints[port_index].HintDescriptor))
         {
@@ -165,6 +154,44 @@ create_int_control (plugin_desc_t * desc, unsigned long port_index)
   return widget;
 }
 
+/** create a control for points ports */
+static GtkWidget *
+create_points_control (plugin_desc_t * desc, unsigned long port_index)
+{
+  GtkWidget *widget = NULL;
+
+#ifdef HAVE_LRDF
+  GtkListStore *store;
+  lrdf_defaults *defs;
+  int i;
+  GtkCellRenderer *renderer;
+
+  store = gtk_list_store_new (2, G_TYPE_STRING, G_TYPE_FLOAT);
+  defs = lrdf_get_scale_values (desc->id, port_index);
+  for (i = 0; i < defs->count; i++) {
+    GtkTreeIter iter;
+
+    gtk_list_store_append (store, &iter);
+    gtk_list_store_set (store, &iter,
+                        0, defs->items[i].label,
+                        1, defs->items[i].value,
+                        -1);
+  }
+  lrdf_free_setting_values (defs);
+
+  widget = gtk_combo_box_new_with_model (GTK_TREE_MODEL (store));
+  g_object_unref (store);
+
+  renderer = gtk_cell_renderer_text_new ();
+  gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (widget), renderer, TRUE);
+  gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (widget), renderer,
+                                  "text", 0,
+                                  NULL);
+#endif
+
+  return widget;
+}
+
 /** creates a string for the a control port label, possibly splitting
     from "foo (bar=baz,foobar=foobaz)" to "foo\nbar=baz\nfoobar=foobaz" */
 static const char *
@@ -318,6 +345,17 @@ plugin_slot_create_control_table_row (plugin_slot_t * plugin_slot, port_controls
           port_controls->controls[i].text = NULL;
           break;
 
+        case JR_CTRL_POINTS:
+          port_controls->controls[i].control =
+            create_points_control (desc, port_controls->port_index);
+          g_signal_connect (G_OBJECT
+                            (port_controls->controls[i].control),
+                            "changed",
+                            G_CALLBACK (control_points_cb),
+                            port_controls);
+          port_controls->controls[i].text = NULL;
+          break;
+
         case JR_CTRL_BOOL:
           port_controls->controls[i].control =
             gtk_toggle_button_new_with_label (_("On"));
@@ -357,7 +395,7 @@ port_controls_new	(plugin_slot_t * plugin_slot)
   desc = plugin_slot->plugin->desc;
 
   /* make the controls array */
-  port_controls_array = g_malloc (sizeof (port_controls_t) * desc->control_port_count);
+  port_controls_array = g_malloc_n (desc->control_port_count, sizeof (port_controls_t));
 
   /* contruct the control rows */
   for (i = 0; i < desc->control_port_count; i++)
@@ -371,6 +409,11 @@ port_controls_new	(plugin_slot_t * plugin_slot)
       port_controls->port_index = desc->control_port_indicies[i];
 
       /* get the port control type from the hints */
+#ifdef HAVE_LRDF
+      if (lrdf_get_scale_values(desc->id, port_controls->port_index) != NULL)
+        port_controls->type = JR_CTRL_POINTS;
+      else
+#endif
       if (LADSPA_IS_HINT_TOGGLED (desc->port_range_hints[port_controls->port_index].HintDescriptor))
         port_controls->type = JR_CTRL_BOOL;
       else if (LADSPA_IS_HINT_INTEGER (desc->port_range_hints[port_controls->port_index].HintDescriptor))
@@ -383,7 +426,7 @@ port_controls_new	(plugin_slot_t * plugin_slot)
       else
         port_controls->logarithmic = FALSE;
       
-      port_controls->controls = g_malloc (sizeof (controls_t) * plugin_slot->plugin->copies);
+      port_controls->controls = g_malloc_n (plugin_slot->plugin->copies, sizeof (controls_t));
       
       plugin_slot_create_control_table_row (plugin_slot, port_controls);
     }
@@ -410,7 +453,7 @@ port_control_set_locked (port_controls_t *port_controls, gboolean locked)
     {
       midi_ctrl = (midi_control_t *) list->data;
       
-      if (midi_ctrl->ladspa_control &&
+      if (midi_ctrl->ctrl_type == LADSPA_CONTROL &&
           midi_ctrl->control.ladspa.control == port_controls->control_index)
         midi_control_set_locked (midi_ctrl, locked);
     }
@@ -445,6 +488,13 @@ port_controls_block_int_callback (port_controls_t * port_controls, guint copy)
             G_CALLBACK (control_int_cb));
 }
 
+void
+port_controls_block_points_callback (port_controls_t * port_controls, guint copy)
+{
+  gtk_widget_block_signal(port_controls->controls[copy].control, "changed",
+            G_CALLBACK (control_points_cb));
+}
+
 void
 port_controls_block_bool_callback (port_controls_t * port_controls, guint copy)
 {
@@ -479,6 +529,13 @@ port_controls_unblock_int_callback (port_controls_t * port_controls, guint copy)
             G_CALLBACK (control_int_cb));
 }
 
+void
+port_controls_unblock_points_callback (port_controls_t * port_controls, guint copy)
+{
+  gtk_widget_unblock_signal (port_controls->controls[copy].control, "changed",
+            G_CALLBACK (control_points_cb));
+}
+
 void
 port_controls_unblock_bool_callback (port_controls_t * port_controls, guint copy)
 {
@@ -502,7 +559,7 @@ port_control_send_midi_value (port_controls_t *port_controls, guint copy, LADSPA
     {
       midi_ctrl = list->data;
       
-      if (midi_ctrl->ladspa_control &&
+      if (midi_ctrl->ctrl_type == LADSPA_CONTROL &&
           midi_ctrl->control.ladspa.control == port_controls->control_index &&
           midi_ctrl->control.ladspa.copy == copy)
         {
