@@ -19,6 +19,11 @@
  */
 
 #include <jack/jack.h>
+
+#if HAVE_JACK_SESSION
+#include <jack/session.h>
+#endif
+
 #include <glib.h>
 #include <stdio.h>
 #include <string.h>
@@ -154,7 +159,7 @@ int process_control_messages (process_info_t * procinfo)
         
       case CTRLMSG_QUIT:
         quitting = 1;
-        err = lff_write (procinfo->process_to_ui, &ctrlmsg);
+        lff_write (procinfo->process_to_ui, &ctrlmsg);
         return 1;
 
       default:
@@ -453,13 +458,25 @@ process_info_connect_jack (process_info_t * procinfo, ui_t * ui)
 {
   _update_status ( _("Connecting to JACK server with client name '%s'"), jack_client_name );
 
-  if (!(procinfo->jack_client = jack_client_new (jack_client_name)))
+#if HAVE_JACK_SESSION
+  if (strlen (session_uuid->str))
+    procinfo->jack_client = jack_client_open (jack_client_name, JackSessionID, NULL, session_uuid->str);
+  else
+#endif
+    procinfo->jack_client = jack_client_open (jack_client_name, JackNullOption, NULL);
+
+  if (!procinfo->jack_client)
         return -1;
   
   _update_status ( _status_cb_data, _("Connected to JACK server") );
 
   jack_set_process_callback (procinfo->jack_client, process, procinfo);
   jack_on_shutdown (procinfo->jack_client, jack_shutdown_cb, ui); /* FIXME: need a generic callback for this, too */
+
+#if HAVE_JACK_SESSION
+  if( jack_set_session_callback )
+	  jack_set_session_callback (procinfo->jack_client, jack_session_cb_aux, ui);
+#endif
                                             
   return 0;
 }
@@ -524,11 +541,11 @@ process_info_set_port_count (process_info_t * procinfo, ui_t * ui, unsigned long
   
   if (procinfo->port_count == 0)
     {
-      procinfo->jack_input_ports = g_malloc (sizeof (jack_port_t *) * port_count);
-      procinfo->jack_output_ports = g_malloc (sizeof (jack_port_t *) * port_count);
+      procinfo->jack_input_ports = g_malloc_n (port_count, sizeof (jack_port_t *));
+      procinfo->jack_output_ports = g_malloc_n (port_count, sizeof (jack_port_t *));
       
-      procinfo->jack_input_buffers = g_malloc (sizeof (LADSPA_Data *) * port_count);
-      procinfo->jack_output_buffers = g_malloc (sizeof (LADSPA_Data *) * port_count);
+      procinfo->jack_input_buffers = g_malloc_n (port_count, sizeof (LADSPA_Data *));
+      procinfo->jack_output_buffers = g_malloc_n (port_count, sizeof (LADSPA_Data *));
     }
   else
     {
@@ -638,6 +655,10 @@ process_info_new (ui_t* ui, unsigned long rack_channels)
   
   jack_set_process_callback (procinfo->jack_client, process, procinfo);
   jack_on_shutdown (procinfo->jack_client, jack_shutdown_cb, ui);
+#if HAVE_JACK_SESSION
+  if( jack_set_session_callback )
+	  jack_set_session_callback (procinfo->jack_client, jack_session_cb_aux, ui);
+#endif
   
   procinfo->ui_to_process = ui->ui_to_process; 
   procinfo->process_to_ui = ui->process_to_ui; 
