@@ -31,10 +31,6 @@
 #include <gtk/gtk.h>
 #include <ladspa.h>
 
-#ifdef HAVE_LADCCA
-#include <ladcca/ladcca.h>
-#endif
-
 #include "ui.h"
 #include "control_message.h"
 #include "ui_callbacks.h"
@@ -52,20 +48,16 @@ ui_init_gui_menu (ui_t * ui, GtkWidget * main_box)
   GtkWidget *file_menuitem;
   
   GtkWidget *file_menu;
-#ifdef HAVE_XML
   GtkWidget *save;
   GtkWidget *save_as;
-#endif /* HAVE_XML */
   GtkWidget *separator;
   GtkWidget *quit;
   GtkWidget *rack_menuitem;
   GtkWidget *rack_menu;
   
-#ifdef HAVE_GNOME
   GtkWidget *help_menuitem;
   GtkWidget *help_menu;
   GtkWidget *about;
-#endif
 
   /* the menu bar */
   menubar_handle = gtk_handle_box_new ();
@@ -83,13 +75,10 @@ ui_init_gui_menu (ui_t * ui, GtkWidget * main_box)
   rack_menuitem = gtk_menu_item_new_with_label (_("Rack"));
   gtk_widget_show (rack_menuitem);
   gtk_menu_shell_append (GTK_MENU_SHELL (menubar), rack_menuitem);
-  
 
-#ifdef HAVE_GNOME  
   help_menuitem = gtk_menu_item_new_with_label (_("Help"));
   gtk_widget_show (help_menuitem);
   gtk_menu_shell_append (GTK_MENU_SHELL (menubar), help_menuitem);
-#endif
 
   /* file menu */
   file_menu = gtk_menu_new ();
@@ -101,7 +90,6 @@ ui_init_gui_menu (ui_t * ui, GtkWidget * main_box)
   g_signal_connect (G_OBJECT (ui->new_menuitem), "activate",
                     G_CALLBACK (new_cb), ui);
 
-#ifdef HAVE_XML
   ui->open_menuitem = gtk_image_menu_item_new_from_stock (GTK_STOCK_OPEN, NULL);
   gtk_widget_show (ui->open_menuitem);
   gtk_menu_shell_append (GTK_MENU_SHELL (file_menu), ui->open_menuitem);
@@ -119,22 +107,21 @@ ui_init_gui_menu (ui_t * ui, GtkWidget * main_box)
   gtk_menu_shell_append (GTK_MENU_SHELL (file_menu), save_as);
   g_signal_connect (G_OBJECT (save_as), "activate",
                     G_CALLBACK (save_as_cb), ui);
-#endif /* HAVE_XML */
 
-#ifdef HAVE_LADCCA
-  if (cca_enabled (global_cca_client))
+#ifdef HAVE_LASH
+  if (lash_enabled (global_lash_client))
     {
       separator = gtk_separator_menu_item_new ();
       gtk_widget_show (separator);
       gtk_menu_shell_append (GTK_MENU_SHELL (file_menu), separator);
 
-      ui->cca_save_menu_item = gtk_menu_item_new_with_label (_("Save project"));
-      gtk_widget_show (ui->cca_save_menu_item);
-      gtk_menu_shell_append (GTK_MENU_SHELL (file_menu), ui->cca_save_menu_item);
-      g_signal_connect (G_OBJECT (ui->cca_save_menu_item), "activate",
-                        G_CALLBACK (cca_save_cb), ui);
+      ui->lash_save_menu_item = gtk_menu_item_new_with_label (_("Save project"));
+      gtk_widget_show (ui->lash_save_menu_item);
+      gtk_menu_shell_append (GTK_MENU_SHELL (file_menu), ui->lash_save_menu_item);
+      g_signal_connect (G_OBJECT (ui->lash_save_menu_item), "activate",
+                        G_CALLBACK (lash_save_cb), ui);
     }
-#endif /* HAVE_LADCCA */
+#endif /* HAVE_LASH */
 
   separator = gtk_separator_menu_item_new ();
   gtk_widget_show (separator);
@@ -172,7 +159,6 @@ ui_init_gui_menu (ui_t * ui, GtkWidget * main_box)
                     G_CALLBACK (midi_cb), ui);
 #endif /* HAVE_ALSA */
 
-#ifdef HAVE_GNOME
   /* help menu */
   help_menu = gtk_menu_new ();
   gtk_menu_item_set_submenu (GTK_MENU_ITEM(help_menuitem), help_menu);
@@ -182,19 +168,17 @@ ui_init_gui_menu (ui_t * ui, GtkWidget * main_box)
   gtk_menu_shell_append (GTK_MENU_SHELL (help_menu), about);
   g_signal_connect (G_OBJECT (about), "activate",
                     G_CALLBACK (about_cb), ui);
-#endif
 }
 
 static void
-ui_set_default_window_icon ()
+ui_set_default_window_icon (void)
 {
-  gchar * icon_file;
+  const char * icon_file = PIXMAPDIR "/" JACK_RACK_ICON_FILE;
   GError * icon_error = NULL;
   GList * icons = NULL;
   GdkPixbuf * icon;
 
   /* set the icons */
-  icon_file = g_strdup_printf ("%s/pixmaps/%s", JR_DESKTOP_PREFIX, JACK_RACK_ICON_FILE);
   icon = gdk_pixbuf_new_from_file (icon_file, &icon_error);
   if (icon_error)
     {
@@ -202,7 +186,6 @@ ui_set_default_window_icon ()
                __FUNCTION__, icon_file, icon_error->message);
       g_error_free (icon_error);
     }
-  g_free (icon_file);
   
   icons = g_list_append (icons, icon);
 
@@ -212,6 +195,7 @@ ui_set_default_window_icon ()
   g_object_unref (icon);
 }
 
+
 static void
 ui_init_gui (ui_t * ui, unsigned long channels)
 {
@@ -222,6 +206,7 @@ ui_init_gui (ui_t * ui, unsigned long channels)
   GtkWidget *plugin_viewport;
   GtkWidget *channel_icon;
   gchar *channel_icon_file;
+  GtkToolItem *tool_item;
 
 
   /* main window */
@@ -246,81 +231,69 @@ ui_init_gui (ui_t * ui, unsigned long channels)
   gtk_box_pack_start (GTK_BOX (main_box), toolbar_handle, FALSE, FALSE, 0);
 
   toolbar = gtk_toolbar_new ();
-  gtk_widget_show (toolbar);
   gtk_container_add (GTK_CONTAINER (toolbar_handle), toolbar);
 
-
-  ui->add = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-				      GTK_STOCK_ADD,
-				      _("Add a plugin"),
-				      (const char *) NULL,
-				      (GtkSignalFunc) NULL,
-				      (gpointer)NULL,
-				      (gint)-1);
-  g_signal_connect_swapped (G_OBJECT (ui->add), "event",
-                            G_CALLBACK (plugin_button_cb),
-                            G_OBJECT (ui->add_menu));
-                                                         
+  ui->add = gtk_menu_tool_button_new_from_stock (GTK_STOCK_ADD);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), ui->add, -1);
+  gtk_tool_item_set_tooltip_text (ui->add, _("Add a plugin"));
+  ui->toolbar_add_menu = plugin_mgr_get_menu (ui->plugin_mgr, G_CALLBACK (add_cb), ui);
+  gtk_menu_tool_button_set_menu (GTK_MENU_TOOL_BUTTON (ui->add), ui->toolbar_add_menu);
+  g_signal_connect (G_OBJECT (ui->add), "clicked",
+                    G_CALLBACK (plugin_add_button_cb), ui->add_menu);
 
   /* channels */
   channel_icon_file = g_strdup_printf ("%s/%s", PKGDATADIR, JACK_RACK_CHANNELS_ICON_FILE);
   channel_icon = gtk_image_new_from_file (channel_icon_file);
   g_free (channel_icon_file);
-  ui->channels = gtk_toolbar_append_item (GTK_TOOLBAR (toolbar),
-                           _("Channels"),
-                           _("Change the number of I/O channels the rack has"),
-                           NULL,
-                           channel_icon,
-                           G_CALLBACK (channel_cb), ui);
-                              
-  
-  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
+  ui->channels = gtk_tool_button_new (channel_icon, _("Channels"));
+  gtk_tool_item_set_tooltip_text (ui->channels, _("Change the number of I/O channels the rack has"));
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), ui->channels, -1);
+  g_signal_connect (G_OBJECT (ui->channels), "clicked", G_CALLBACK (channel_cb), ui);
 
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), gtk_separator_tool_item_new (), -1);
 
-  ui->new = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_NEW,
-                            _("Clear the rack"),
-                            NULL, G_CALLBACK (new_cb), ui, -1);
+  ui->new = gtk_tool_button_new_from_stock (GTK_STOCK_NEW);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), ui->new, -1);
+  gtk_tool_item_set_tooltip_text (ui->new, _("Clear the rack"));
+  g_signal_connect (G_OBJECT (ui->new), "clicked", G_CALLBACK (new_cb), ui);
 
-#ifdef HAVE_XML
-  ui->open = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_OPEN,
-                            _("Open a rack configuration file"),
-                            NULL, G_CALLBACK (open_cb), ui, -1);
+  tool_item = gtk_tool_button_new_from_stock (GTK_STOCK_OPEN);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), tool_item, -1);
+  gtk_tool_item_set_tooltip_text (tool_item, _("Load a previously-saved rack configuration"));
+  g_signal_connect (G_OBJECT (tool_item), "clicked", G_CALLBACK (open_cb), ui);
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_SAVE,
-                            _("Save the rack configuration to the current file"),
-                            NULL, G_CALLBACK (save_cb), ui, -1);
+  tool_item = gtk_tool_button_new_from_stock (GTK_STOCK_SAVE);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), tool_item, -1);
+  gtk_tool_item_set_tooltip_text (tool_item, _("Save the rack configuration to the current file"));
+  g_signal_connect (G_OBJECT (tool_item), "clicked", G_CALLBACK (save_cb), ui);
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_SAVE_AS,
-                            _("Save the rack configuration to a file"),
-                            NULL, G_CALLBACK (save_as_cb), ui, -1);
-#endif /* HAVE_XML */
+  tool_item = gtk_tool_button_new_from_stock (GTK_STOCK_SAVE_AS);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), tool_item, -1);
+  gtk_tool_item_set_tooltip_text (tool_item, _("Save the rack configuration to a new file"));
+  g_signal_connect (G_OBJECT (tool_item), "clicked", G_CALLBACK (save_as_cb), ui);
 
-#ifdef HAVE_LADCCA
-  if (cca_enabled (global_cca_client))
+#ifdef HAVE_LASH
+  if (lash_enabled (global_lash_client))
     {
-      gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
+      gtk_toolbar_insert (GTK_TOOLBAR (toolbar), gtk_separator_tool_item_new (), -1);
+
+      ui->lash_save = gtk_tool_button_new_from_stock (GTK_STOCK_SAVE);
+      gtk_toolbar_insert (GTK_TOOLBAR (toolbar), ui->lash_save, -1);
+      gtk_tool_item_set_tooltip_text (ui->lash_save, _("Save the current LASH project"));
+      g_signal_connect (G_OBJECT (ui->lash_save), "clicked", G_CALLBACK (lash_save_cb), ui);
 
-      ui->cca_save = gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                                           GTK_STOCK_SAVE,
-                                           _("Save the LADCCA project"),
-                                           NULL, G_CALLBACK (cca_save_cb), ui, -1);
     }
+#endif /* HAVE_LASH */
 
-#endif /* HAVE_LADCCA */
-  gtk_toolbar_append_space (GTK_TOOLBAR (toolbar));
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), gtk_separator_tool_item_new (), -1);
 
-  gtk_toolbar_insert_stock (GTK_TOOLBAR (toolbar),
-                            GTK_STOCK_QUIT,
-                            _("Quit JACK Rack"),
-                            NULL, G_CALLBACK (quit_cb), ui, -1);
+  tool_item = gtk_tool_button_new_from_stock (GTK_STOCK_QUIT);
+  gtk_toolbar_insert (GTK_TOOLBAR (toolbar), tool_item, -1);
+  gtk_tool_item_set_tooltip_text (tool_item, _("Quit JACK Rack"));
+  g_signal_connect (G_OBJECT (tool_item), "clicked", G_CALLBACK (quit_cb), ui);
 
-  gtk_toolbar_set_icon_size (GTK_TOOLBAR (toolbar), GTK_ICON_SIZE_SMALL_TOOLBAR);
+  gtk_widget_show_all (toolbar);
 
-                              
 
   /* viewport thingy */
   plugin_scroll = gtk_scrolled_window_new (NULL, NULL);
@@ -353,7 +326,6 @@ ui_init_gui (ui_t * ui, unsigned long channels)
   gtk_menu_shell_append (GTK_MENU_SHELL (ui->midi_menu), ui->midi_menu_item);
 #endif /* HAVE_ALSA */
 
-#ifdef HAVE_XML
   /* open file from command line, if any */
   if ( initial_filename != NULL )
   {
@@ -362,11 +334,12 @@ ui_init_gui (ui_t * ui, unsigned long channels)
   
     if (!err)
       ui_set_filename (ui, fn);
-    /* free? */
+
+    g_free (fn);
   }
-#endif /* HAVE_XML */
 }
 
+
 void
 ui_init_splash_screen (ui_t * ui)
 {
@@ -383,7 +356,6 @@ ui_init_splash_screen (ui_t * ui)
   gtk_window_set_resizable (GTK_WINDOW (ui->splash_screen), FALSE);
   gtk_window_set_title (GTK_WINDOW (ui->splash_screen), PACKAGE_NAME);
   gtk_container_set_resize_mode (GTK_CONTAINER (ui->splash_screen), GTK_RESIZE_IMMEDIATE);
-  gtk_window_set_policy (GTK_WINDOW (ui->splash_screen), FALSE, TRUE, FALSE);
   
   box = gtk_vbox_new (FALSE, 0);
   gtk_container_set_border_width (GTK_CONTAINER (box), 3);
@@ -398,8 +370,8 @@ ui_init_splash_screen (ui_t * ui)
   ui->splash_screen_text = gtk_label_new (NULL);
   gtk_widget_show (ui->splash_screen_text);
   gtk_box_pack_start (GTK_BOX (box), ui->splash_screen_text, TRUE, TRUE, 0);
-  //  gtk_misc_set_alignment (GTK_MISC (ui->splash_screen_text), 0.0, 0.5);
   gtk_label_set_justify (GTK_LABEL (ui->splash_screen_text), GTK_JUSTIFY_CENTER);
+  gtk_label_set_ellipsize (GTK_LABEL (ui->splash_screen_text), PANGO_ELLIPSIZE_END);
   
   gtk_window_get_size (GTK_WINDOW (ui->splash_screen), &w, &h);
   gtk_window_move (GTK_WINDOW (ui->splash_screen),
@@ -408,6 +380,7 @@ ui_init_splash_screen (ui_t * ui)
   gtk_window_set_position (GTK_WINDOW (ui->splash_screen), GTK_WIN_POS_CENTER_ALWAYS);
   gtk_widget_show (ui->splash_screen);
 
+  /* FIXME: now what a dirty hack is this? */
   for (i = 0; i < 20; i++)
     gtk_main_iteration_do (FALSE);
 }
@@ -415,7 +388,7 @@ ui_init_splash_screen (ui_t * ui)
 ui_t *
 ui_new (unsigned long channels)
 {
-  ui_t * ui;
+  ui_t* ui;
 
   ui = g_malloc (sizeof (ui_t));
   ui->splash_screen = NULL;
@@ -425,23 +398,37 @@ ui_new (unsigned long channels)
   ui->state = STATE_NORMAL;
   ui->main_window = NULL;
 
-#ifdef HAVE_LADCCA  
-  ui->cca_save = NULL;
-  ui->cca_save_menu_item = NULL;
+#ifdef HAVE_LASH  
+  ui->lash_save = NULL;
+  ui->lash_save_menu_item = NULL;
 #endif
 
   ui->ui_to_process = lff_new (PROCESS_FIFO_SIZE, sizeof (ctrlmsg_t));
   ui->process_to_ui = lff_new (PROCESS_FIFO_SIZE, sizeof (ctrlmsg_t));
+
 #ifdef HAVE_ALSA
   ui->ui_to_midi    = lff_new (MIDI_FIFO_SIZE, sizeof (ctrlmsg_t));
   ui->midi_to_ui    = lff_new (MIDI_FIFO_SIZE, sizeof (ctrlmsg_t));
 #endif
 
   ui_init_splash_screen (ui);
-  
-  ui->procinfo = process_info_new (ui, channels);
+
+  ui->procinfo = process_info_new(ui, channels);
+
   if (!ui->procinfo)
-    return NULL;
+  {
+          /* FIXME: show message, free lffs */
+          g_free(ui);
+          return NULL;
+  }
+
+  if (ui->state == STATE_QUITTING)
+  {
+          /* connect failed */
+          g_free(ui);
+          /* FIXME free lffs */
+          return NULL;
+  }
 
 #ifdef HAVE_ALSA
   ui->midi_info   = midi_info_new (ui);
@@ -450,7 +437,7 @@ ui_new (unsigned long channels)
   plugin_mgr_set_plugins (ui->plugin_mgr, channels);
   ui->jack_rack = jack_rack_new (ui, channels);
 
-  gtk_idle_add (idle_cb, ui);
+  g_idle_add (idle_cb, ui);
 
   ui_init_gui (ui, channels);
 
@@ -479,13 +466,22 @@ ui_destroy (ui_t * ui)
   g_free (ui);
 }
 
+
+/**
+ * Display a message box with an error message.
+ *
+ * @todo Use severity_t.
+ *
+ * @param data A ui_t*
+ */
 void
-ui_display_error (ui_t * ui, const char * format, ...)
+ui_display_error (void* data, error_severity_t severity, const char * format, ...)
 {
   va_list args;
+  ui_t* ui = (ui_t*)data;
   gchar * text;
   GtkWidget * dialog;
-  
+
   va_start(args, format);
   
   text = g_strdup_vprintf (format, args);
@@ -504,10 +500,17 @@ ui_display_error (ui_t * ui, const char * format, ...)
   va_end(args);
 }
 
+
+/**
+ * Display a text message in the splash window
+ *
+ * @param data A ui_t*
+ */
 void
-ui_display_splash_text (ui_t * ui, const char * format, ...)
+ui_display_splash_text (void* data, const char* format, ...)
 {
   va_list args;
+  ui_t* ui = (ui_t*)data; 
   gchar * text;
   short j;
   
@@ -589,12 +592,14 @@ ui_set_state    (ui_t * ui, ui_state_t state)
   ui->state = state;
 }
 
+
 ui_state_t
 ui_get_state (ui_t * ui)
 {
   return ui->state;
 }
 
+
 void
 ui_set_channels (ui_t * ui, unsigned long channels)
 {
@@ -642,8 +647,3 @@ ui_set_channels (ui_t * ui, unsigned long channels)
   
   jack_activate (ui->procinfo->jack_client);
 }
-
-/* EOF */
-
-
-
