@@ -27,13 +27,10 @@
 #include <math.h>
 #include <unistd.h>
 
+#include <glib.h>
 #include <gtk/gtk.h>
 #include <ladspa.h>
 
-#ifdef HAVE_GNOME
-#include <libgnomeui/libgnomeui.h>
-#endif
-
 #include "jack_rack.h"
 #include "lock_free_fifo.h"
 #include "ui_callbacks.h"
@@ -123,7 +120,7 @@ new_cb (GtkWidget * widget, gpointer user_data)
   ui_set_filename (ui, NULL);
 }
 
-#ifdef HAVE_XML
+
 void
 open_cb (GtkButton * button, gpointer user_data)
 {
@@ -189,8 +186,8 @@ save_as_cb (GtkButton * button, gpointer user_data)
 
   gtk_widget_destroy (dialog);
 }
-#endif /* HAVE_XML */
- 
+
+
 #ifdef HAVE_LASH
 void
 lash_save_cb (GtkButton * button, gpointer user_data)
@@ -235,7 +232,7 @@ window_destroy_cb (GtkWidget *widget,
   return TRUE;
 }
 
-#ifdef HAVE_GNOME
+
 void
 about_cb (GtkWidget * widget, gpointer user_data)
 {
@@ -266,7 +263,6 @@ about_cb (GtkWidget * widget, gpointer user_data)
                          NULL);
 
 }
-#endif /* HAVE_GNOME */
 
 
 #ifdef HAVE_ALSA
@@ -317,13 +313,18 @@ lash_idle (ui_t * ui, lash_client_t * client)
       switch (lash_event_get_type (event))
         {
         case LASH_Save_File:
-          ui_save_file (ui, lash_get_fqn (lash_event_get_string (event), "jack_rack.rack"));
-          lash_send_event (global_lash_client, event);
-          break;
         case LASH_Restore_File:
-          ui_open_file (ui, lash_get_fqn (lash_event_get_string (event), "jack_rack.rack"));
-          lash_send_event (global_lash_client, event);
-          break;
+          {
+            gchar *filename = g_strjoin ("/", lash_event_get_string (event), "jack_rack.rack", NULL);
+            if (lash_event_get_type (event) == LASH_Save_File)
+              ui_save_file (ui, filename);
+            else
+              ui_open_file (ui, filename);
+            g_free (filename);
+
+            lash_send_event (global_lash_client, event);
+            break;
+          }
         case LASH_Quit:
           quit_cb (NULL, ui);
           lash_event_destroy (event);
@@ -458,7 +459,7 @@ reconnect_cb ( gpointer data )
 {
         struct reconnect_data*  rcdata = (struct reconnect_data*)data;
 
-        process_info_t*         procinfo = rcdata->ui->procinfo;
+        process_info_t*         procinfo;
         
         if ( rcdata->active == FALSE )
                 return FALSE;
@@ -483,11 +484,9 @@ void
 setup_reconnect ( gpointer data )
 {
         static GtkWidget* dialog = NULL;
-        static int active = TRUE;
         static struct reconnect_data rcdata;
-	guint tid;
-	gint answer;
-	GtkWidget *rcnotice;
+        gint answer;
+        GtkWidget *rcnotice;
 
         ui_t* ui = (ui_t*)data;
         
@@ -507,7 +506,7 @@ setup_reconnect ( gpointer data )
         gtk_widget_show_all (GTK_DIALOG(dialog)->vbox);
         rcdata.dialog = dialog;
         
-        tid = g_timeout_add (1000, &reconnect_cb, (gpointer)&rcdata);
+        g_timeout_add (1000, &reconnect_cb, (gpointer)&rcdata);
         
         answer = gtk_dialog_run (GTK_DIALOG(dialog));
         
@@ -533,13 +532,10 @@ setup_reconnect ( gpointer data )
                 gtk_widget_set_sensitive (ui->midi_menuitem, FALSE);
                 gtk_widget_set_sensitive (ui->midi_window->main_box, FALSE);
 #endif
-#ifdef HAVE_XML
                 gtk_widget_set_sensitive (GTK_WIDGET (ui->open), FALSE);
                 gtk_widget_set_sensitive (ui->open_menuitem, FALSE);
-#endif
         }
         
-        active = FALSE;
         gtk_widget_hide (dialog);
 }
 
@@ -655,3 +651,40 @@ jack_shutdown_cb (void * data)
   ui->shutdown = TRUE;
 }
 
+#if HAVE_JACK_SESSION 
+
+static gboolean
+jack_session_cb (gpointer data)
+{
+  ui_t * ui = (ui_t*)data;
+  char cmd_buf[256];
+  char fname_buf[256];
+
+  snprintf( fname_buf, sizeof(fname_buf), "%srack.xml", ui->js_event->session_dir);
+  snprintf( cmd_buf, sizeof(cmd_buf), "jack-rack --jack-session-uuid %s \"${SESSION_DIR}rack.xml\"", ui->js_event->client_uuid );
+
+  ui->js_event->command_line = strdup( cmd_buf );
+
+  if (!ui_save_file (ui, fname_buf))
+    ui->js_event->flags = JackSessionSaveError;
+
+  jack_session_reply( ui->procinfo->jack_client, ui->js_event );
+
+  if (ui->js_event->type == JackSessionSaveAndQuit)
+    quit_cb (NULL, ui);
+
+  jack_session_event_free( ui->js_event );
+
+  return FALSE;
+}
+
+void 
+jack_session_cb_aux (jack_session_event_t *ev, void *data)
+{
+  ui_t * ui = (ui_t*)data;
+  ui->js_event = ev;
+  g_idle_add (jack_session_cb, data);
+}
+
+#endif
+
